<?php

namespace Addons\DyyEmail\Http\Controllers;

use Addons\DyyEmail\Constant\Constant;
use Addons\DyyEmail\Http\Requests\CompanyRequest;
use Addons\DyyEmail\Http\Requests\UserRequest;
use Addons\DyyEmail\Models\Dyyemail;
use App\Modules\Admin\Models\User;
use Addons\DyyEmail\Collections\CommonResourceCollection;
use App\Modules\Common\Controllers\BaseController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use GuzzleHttp;

class DyyEmailController extends BaseController
{
    public function __construct(Dyyemail $dyyemail)
    {
        $this->model = $dyyemail;
        $this->modelName = 'Dyyemail';
        $this->viewPath = Constant::APP_SIGN . '::dyyemail.';
        $this->validatorData = [
            'name' => 'required'
        ];
        $this->validatorMessages = [
            'url.url' => '请填写url链接',
            'url.unique' => '该链接已存在'
        ];

        $this->client = new GuzzleHttp\Client(['base_uri' => 'https://dyyemail.com', 'verify' => false, 'timeout' => 60]);

    }

    public function index()
    {
        $request = \request();
        $user = $request->user();
        if ($request->ajax() || $request->wantsJson()) {
            $dyyemailAdmin = $this->model->where('role_id', 1)->oldest()->first();
            $data = tap($this->model->with(['user'])->oldest(), function ($query) use ($dyyemailAdmin, $user) {
                if ($user->id !== 1 && $user->id !== $dyyemailAdmin->user_id) {
                    $query->where('user_id', $user->id);
                }
            })->paginate($request->input('limit', 15));
            if ($collection = $this->collection) {
                return new $collection($data);
            } else {
                return new CommonResourceCollection($data);

            }
        }
        $count = $this->model->count();
        if (!$count) {
            $admins = User::query()->get();
            return view($this->viewPath . '.kaihu', compact('admins'));
        } else {
            $dyyemail = $this->model->where('user_id', $user->id)->first() ?: $this->model;
            return view($this->viewPath . '.index', compact('dyyemail', 'user'));
        }

    }

    public function create()
    {
        $user = \request()->user();
        $dyyemail = Dyyemail::query()->where('role_id', 1)->oldest()->firstOrNew();
        $allUserIds = array_merge(Dyyemail::pluck('user_id')->toArray(), [1]);
        if ($user->id === 1 || $dyyemail->user_id === $user->id) {
            $admins = User::query()->get();
            return view($this->viewPath . '.create', compact('allUserIds', 'admins'));
        }
    }

    public function login(Request $request, $id)
    {
        $user = $request->user();
        $dyyemail = Dyyemail::query()->where('role_id', 1)->oldest()->firstOrNew();
        if ($user->id === 1 || $dyyemail->user_id === $user->id) {
            try {
                $dyyemail = Dyyemail::query()->findOrFail($id);
                $response = $this->client->post('/api/youhuatong/getLoginTemp', [
                    'form_params' => [
                        'token' => app('settings')['setting']->website_token,
                        'user_id' => $dyyemail->dyyemail_id
                    ]
                ]);
                $result = json_decode((string)$response->getBody(), 1);
                if ($result['status']) {
                    return redirect($result['data']);
                } else {
                    return $this->badRequest($result['err_msg']);
                }
            } catch (\Exception $exception) {
                Log::info('Dyyemail登陆失败，原因为：' . $exception->getMessage());
                dd('登陆失败，请联系第一页开发人员');
            }

        } else {
            dd('暂无权限');
        }
    }

    public function user(UserRequest $request)
    {
        $dyyemailAdmin = Dyyemail::query()->where('role_id', 1)->oldest()->firstOrFail();
        try {
            DB::beginTransaction();
            $add = [];
            $add['user_id'] = $request->get('user_id');
            $add['role_id'] = 0;
            $add['name'] = $request->get('name');
            $add['mobile'] = $request->get('mobile');
            $add['email'] = $request->get('email');
            $add['company'] = $dyyemailAdmin->company;
            $add['parent_id'] = $dyyemailAdmin->user_id;
            $dyyemail = Dyyemail::create($add);
            $response = $this->client->post('/api/youhuatong/createUser', [
                'form_params' => [
                    'token' => app('settings')['setting']->website_token,
                    'name' => $dyyemail->name,
                    'mobile' => $dyyemail->mobile,
                    'email' => $dyyemail->email,
                    'company' => $dyyemail->company,
                    'password' => $request->get('password'),
                    'parent_id' => $dyyemailAdmin->dyyemail_id
                ]
            ]);
            Log::info($response->getStatusCode());
            Log::info((string)$response->getBody());
            $result = json_decode((string)$response->getBody(), 1);
            if ($result['status']) {
                $dyyemail->dyyemail_id = $result['data'];
                $dyyemail->save();
                DB::commit();
                return $this->success();
            } else {
                DB::rollBack();
                return $this->badRequest($result['err_msg']);
            }

        } catch (\Exception $exception) {
            Log::info('Dyyemail添加用户失败，原因为：' . $exception->getMessage());
            DB::rollBack();
            return $this->badRequest('添加用户失败，请联系第一页开发人员');
        }
    }

    public function getData(Request $request, $name)
    {
        $response = [
            'status' => false
        ];
        if ($request->header('token') === app('settings')['setting']->website_token) {
            $db = false;
            switch ($name) {
                case '优化通newsletter邮箱':
                    $db = DB::table('newsletters');
                    break;
                case '优化通啄木鸟邮箱':
                    $db = DB::table('woodpeckers');
                    break;
                case '优化通Listing推荐线索邮箱':
                    $db = DB::table('inquiries')->where('title', 'like', '%【First Page TL Inquiry】%');
                    break;
                default:
                    $response['error_msg'] = '参数错误，请检查';
                    break;
            }
            if ($db) {
                if ($last_id = $request->get('last_id')) {
                    $model = $db->where('id', '>', $last_id)->orderBy('id')->paginate(500);
                } else {
                    $model = $db->orderBy('id')->paginate(500);
                }
                $response['status'] = true;
                $response['data'] = $model;
            }
        } else {
            $response['error_msg'] = '认证失败!';
        }
        return $response;
    }

    public function company(CompanyRequest $request)
    {
        try {
            DB::beginTransaction();
            $add = [];
            $add['user_id'] = $request->get('user_id');
            $add['role_id'] = 1;
            $add['name'] = $request->get('name');
            $add['mobile'] = $request->get('mobile');
            $add['email'] = $request->get('email');
            $add['company'] = $request->get('company');
            $dyyemail = Dyyemail::create($add);
            $response = $this->client->post('/api/youhuatong/createCompany', [
                'form_params' => array_merge([
                    'token' => app('settings')['setting']->website_token,
                ], $request->all())
            ]);
            $result = json_decode((string)$response->getBody(), 1);
            if ($result['status']) {
                $dyyemail->dyyemail_id = $result['data'];
                $dyyemail->save();
                DB::commit();
                return  $this->data([
                    'status' => true,
                ]);
            } else {
                DB::rollBack();
                return  $this->data([
                    'status' => false,
                    'error_msg' => $result['err_msg']
                ]);
            }
        } catch (\Exception $exception) {
            Log::info('Dyyemail开户失败，原因为：' . $exception->getMessage());
            return  $this->data([
                'status' => false,
                'error_msg' =>'开户失败，请联系第一页开发人员'
            ]);
//            return back()->withErrors('开户失败，请联系第一页开发人员');
//            DB::rollBack();
//            dd('开户失败，请联系第一页开发人员');
        }
    }
}
